#include <common.h>
#include <command.h>
#include <asm/mipsregs.h>
#include <asm/addrspace.h>
#include <config.h>
#include <version.h>
#include "ar7240_soc.h"

#ifdef COMPRESSED_UBOOT
#	define prmsg(...)
#else
#	define prmsg	printf
#endif

/* #define GPIO_DEBUG 1 */

#define GPIO_REGISTER_THRESHOLD 24
#define ADDRESS_UNDEFINE 0

typedef uint32_t gpio_no_t;
typedef enum { GPIO_INPUT,
 			   GPIO_OUTPUT,
			   GPIO_BOTH,
  		     } gpio_direction_t;
			 
typedef enum { GPIO_IGNORE, 
               GPIO_LOW,
			   GPIO_HIGH,
  		     } gpio_action_t;
			 
typedef struct {
	gpio_no_t		  pin;	            /* GPIO number     */
	gpio_direction_t  dir;	            /* Direction       */
	gpio_action_t	  active;           /* active polarity */
	gpio_action_t     init_gpio_value;  /* init value	   */
} alpha_gpio_table_t;

typedef struct {
	gpio_no_t		bit;	            /* bit number      */
	gpio_no_t       bit_value;          /* bit value       */
} alpha_gpio_mode_t;

typedef struct {
    alpha_gpio_table_t *gpio_table;
    alpha_gpio_mode_t *gpio_mode;
    void ( *gpio_init_extra ) ( void );
} alpha_gpio_t;



/****************************************************************************/
#ifdef ALPHA_S17_GIGA /* GPIO Shell User Define, 2011.10.06 Daniel Chen */

#define GPIO_SET_OUTPUT_IS_1 0 /* If your datasheet is set 1 for output, set the value to 1 otherwise set to 0  */

/* define the GPIO's name and number.                                                  */
#define GPIO_WLAN_LED		      11 /* GPIO 11:    WLAN LED                           */
#define GPIO_INTERNET_LED         12 /* GPIO 12:    Internet LED Blue       		   */
#define GPIO_WSP_LED              13 /* GPIO 13:    WPS LED                 		   */
#define GPIO_POWER_LED            14 /* GPIO 14:    Power LED Blue                     */
#define GPIO_EXTERMAL_LNA0        15 /* GPIO 15:    Control signal for external LNA 0  */
#define GPIO_RESET_BUTTON         17 /* GPIO 17:    Factory Reset Button               */
#define GPIO_EXTERMAL_LNA1        18 /* GPIO 18:    Control signal for external LNA 1  */
#define GPIO_WPS_BUTTON           19 /* GPIO 19:    WPS Button                         */


/* Set the corresponding CPU's address, if you do not want to use or not defined please set ADDRESS_UNDEFINE */
typedef enum { CPU_GPIOMODE_REG		 = AR7240_GPIO_FUNC,
			   CPU_GPIO_OUT_VALUE    = AR7240_GPIO_OUT,
			   CPU_GPIO_IN_VALUE     = AR7240_GPIO_IN,
			   CPU_REG_PIODATA	     = ADDRESS_UNDEFINE,
			   CPU_REG_PIODIR		 = AR7240_GPIO_OE,
			   CPU_REG_PIODIR3924	 = ADDRESS_UNDEFINE,
			   CPU_REG_PIOSET        = AR7240_GPIO_SET,
			   CPU_REG_PIORESET      = AR7240_GPIO_CLEAR,
			   CPU_REG_PIOSET3924    = ADDRESS_UNDEFINE,
			   CPU_REG_PIORESET3924  = ADDRESS_UNDEFINE,
			   CPU_REG_PIOTOG        = ADDRESS_UNDEFINE,//RT5350_REG_PIOTOG,
 		     } gpio_pio_maps_t;

			 

alpha_gpio_table_t alpha_gpio_table[] = {
/* Defined GPIO number, direction, activation, and the initial value(pull high/low) you want. */
/*     GPIO number              Direction      active      init_gpio_value */
    {  GPIO_WLAN_LED,           GPIO_OUTPUT,   GPIO_HIGH,  GPIO_LOW     },
    {  GPIO_INTERNET_LED,       GPIO_OUTPUT,   GPIO_HIGH,  GPIO_LOW     },
    {  GPIO_WSP_LED,            GPIO_OUTPUT,   GPIO_HIGH,  GPIO_LOW     },
    {  GPIO_POWER_LED,          GPIO_OUTPUT,   GPIO_HIGH,  GPIO_HIGH    },
    {  GPIO_EXTERMAL_LNA0,      GPIO_OUTPUT,   GPIO_HIGH,  GPIO_LOW     },
    {  GPIO_RESET_BUTTON,       GPIO_INPUT,    GPIO_LOW,   GPIO_HIGH    },	
    {  GPIO_EXTERMAL_LNA1,      GPIO_OUTPUT,   GPIO_HIGH,  GPIO_LOW     },
    {  GPIO_WPS_BUTTON,         GPIO_INPUT,    GPIO_LOW,   GPIO_HIGH    },
};

alpha_gpio_mode_t alpha_gpio_mode_table[] = {
/* set the value of bits to cpu's gpioMode in here */
/*    bits    bit_value */
//    { 1,      1,        },/* disable JTAG port functionality to enable GPIO functionatlity */
};

void gpio_init_extra(void)
{
	/* printf("If you wanted to run other settings, please write here. \n"); */
	
	/* set gpio 11's function value to 0 for GPIO normal model */
	ar7240_reg_wr (AR7240_GPIO_OUT_FUNCTION2,  (ar7240_reg_rd(AR7240_GPIO_OUT_FUNCTION2)&0xffffff ) );
}

#endif


#ifdef ALPHA_S27_MEGA  /* GPIO Shell User Define, 2011.10.06 Daniel Chen */

#define GPIO_SET_OUTPUT_IS_1 0 /* If your datasheet is set 1 for output, set the value to 1 otherwise set to 0  */

/* define the GPIO's name and number.                                       */
#define GPIO_WLAN_LED		       0 /* GPIO  0:    WLAN LED                */
#define GPIO_GREEN_LED_LAN1        1 /* GPIO  1:    LAN1's GREEN LED        */
#define GPIO_GREEN_LED_LAN2        2 /* GPIO  2:    LAN2's GREEN LED        */
#define GPIO_GREEN_LED_LAN3        3 /* GPIO  3:    LAN3's GREEN LED        */
#define GPIO_GREEN_LED_LAN4        4 /* GPIO  4:    LAN4's GREEN LED        */
#define GPIO_POWER_LED            11 /* GPIO 11:    Power LED Blue          */
#define GPIO_INTERNET_LED         12 /* GPIO 12:    Internet LED Blue       */
#define GPIO_WSP_LED              13 /* GPIO 13:    WPS LED                 */
#define GPIO_EXTERMAL_LNA0        14 /* GPIO 14:    Control signal for external LNA 0 */
#define GPIO_EXTERMAL_LNA1        15 /* GPIO 15:    Control signal for external LNA 1 */
#define GPIO_RESET_BUTTON         16 /* GPIO 16:    Factory Reset Button    */
#define GPIO_WPS_BUTTON           17 /* GPIO 17:    WPS Button              */
#define GPIO_YELLOW_LED_LAN1      19 /* GPIO 19:    LAN1's YELLOW LED       */
#define GPIO_YELLOW_LED_LAN2      20 /* GPIO 20:    LAN2's YELLOW LED       */
#define GPIO_YELLOW_LED_LAN3      21 /* GPIO 21:    LAN3's YELLOW LED       */
#define GPIO_YELLOW_LED_LAN4      22 /* GPIO 22:    LAN4's YELLOW LED       */

/* Set the corresponding CPU's address, if you do not want to use or not defined please set ADDRESS_UNDEFINE */
typedef enum { CPU_GPIOMODE_REG		 = AR7240_GPIO_FUNC,
			   CPU_GPIO_OUT_VALUE    = AR7240_GPIO_OUT,
			   CPU_GPIO_IN_VALUE     = AR7240_GPIO_IN,
			   CPU_REG_PIODATA	     = ADDRESS_UNDEFINE,
			   CPU_REG_PIODIR		 = AR7240_GPIO_OE,
			   CPU_REG_PIODIR3924	 = ADDRESS_UNDEFINE,
			   CPU_REG_PIOSET        = AR7240_GPIO_SET,
			   CPU_REG_PIORESET      = AR7240_GPIO_CLEAR,
			   CPU_REG_PIOSET3924    = ADDRESS_UNDEFINE,
			   CPU_REG_PIORESET3924  = ADDRESS_UNDEFINE,
			   CPU_REG_PIOTOG        = ADDRESS_UNDEFINE,//RT5350_REG_PIOTOG,
 		     } gpio_pio_maps_t;

			 

alpha_gpio_table_t alpha_gpio_table[] = {
/* Defined GPIO number, direction, activation, and the initial value(pull high/low) you want. */
/*     GPIO number              Direction       active      init_gpio_value */
    {  GPIO_WLAN_LED,           GPIO_OUTPUT,    GPIO_HIGH,  GPIO_LOW     },
    {  GPIO_GREEN_LED_LAN1,     GPIO_OUTPUT,    GPIO_LOW,   GPIO_HIGH    },
    {  GPIO_GREEN_LED_LAN2,     GPIO_OUTPUT,    GPIO_LOW,   GPIO_HIGH    },
    {  GPIO_GREEN_LED_LAN3,     GPIO_OUTPUT,    GPIO_LOW,   GPIO_HIGH    },
    {  GPIO_GREEN_LED_LAN4,     GPIO_OUTPUT,    GPIO_LOW,   GPIO_HIGH    },
    {  GPIO_POWER_LED,          GPIO_OUTPUT,    GPIO_HIGH,  GPIO_HIGH    },
    {  GPIO_INTERNET_LED,       GPIO_OUTPUT,    GPIO_HIGH,  GPIO_LOW     },
    {  GPIO_WSP_LED,            GPIO_OUTPUT,    GPIO_HIGH,  GPIO_LOW     },
    {  GPIO_EXTERMAL_LNA0,      GPIO_OUTPUT,    GPIO_HIGH,  GPIO_LOW     },
    {  GPIO_EXTERMAL_LNA1,      GPIO_OUTPUT,    GPIO_HIGH,  GPIO_LOW     },
    {  GPIO_RESET_BUTTON,       GPIO_INPUT,     GPIO_LOW,   GPIO_HIGH    },
    {  GPIO_WPS_BUTTON,         GPIO_INPUT,     GPIO_LOW,   GPIO_HIGH    },
    {  GPIO_YELLOW_LED_LAN1,    GPIO_OUTPUT,    GPIO_LOW,   GPIO_HIGH    },
    {  GPIO_YELLOW_LED_LAN2,    GPIO_OUTPUT,    GPIO_LOW,   GPIO_HIGH    },
    {  GPIO_YELLOW_LED_LAN3,    GPIO_OUTPUT,    GPIO_LOW,   GPIO_HIGH    },
    {  GPIO_YELLOW_LED_LAN4,    GPIO_OUTPUT,    GPIO_LOW,   GPIO_HIGH    },
};

alpha_gpio_mode_t alpha_gpio_mode_table[] = {
/* set the value of bits to cpu's gpioMode in here */
/*    bits    bit_value */
    { 1,      1,        },/* disable JTAG port functionality to enable GPIO functionatlity */
};

void gpio_init_extra(void)
{
   /* printf("If you wanted to run other settings, please write here. \n"); */
#if 0
	/* set gpio 4's function value to 0 for GPIO normal model */
	ar7240_reg_wr (AR7240_GPIO_OUT_FUNCTION1,  (ar7240_reg_rd(AR7240_GPIO_OUT_FUNCTION1)&0xffffff00 ) );
	/* set gpio 11's function value to 0 for GPIO normal model */
	ar7240_reg_wr (AR7240_GPIO_OUT_FUNCTION2,  (ar7240_reg_rd(AR7240_GPIO_OUT_FUNCTION2)&0x00ffffff ) );
#endif
	/* Set all define's GPIO OUT FUNCTION to 0 except GPIO_YELLOW_LED_LAN1~4 */
	ar7240_reg_wr (AR7240_GPIO_OUT_FUNCTION0, 0x0 );//port 1 green
	ar7240_reg_wr (AR7240_GPIO_OUT_FUNCTION1,  (ar7240_reg_rd(AR7240_GPIO_OUT_FUNCTION1) &0xffffff00 ) );
	ar7240_reg_wr (AR7240_GPIO_OUT_FUNCTION2,  (ar7240_reg_rd(AR7240_GPIO_OUT_FUNCTION2)&0x00ffffff ) );	
	ar7240_reg_wr (AR7240_GPIO_OUT_FUNCTION3, 0x0 );	
	
	/* set GPIO_GREEN LED_LAN1~4 to LED_LINK mode*/
	ar7240_reg_wr (AR7240_GPIO_OUT_FUNCTION0,  (ar7240_reg_rd(AR7240_GPIO_OUT_FUNCTION0)&0x000000ff ) | ( (42 << 8) | (43 << 16 ) | (44 << 24) ) );//port 1 green
	ar7240_reg_wr (AR7240_GPIO_OUT_FUNCTION1,  (ar7240_reg_rd(AR7240_GPIO_OUT_FUNCTION1) &0xffffff00 ) | ( 45 ) );
}

/* The function is work run for link LED action. 20111021 Daniel Chen */
int green_LED_counter=0, lan_flag[4]={0,0,0,0};
void green_LED_control(void)
{
	int i=0;
	int alive=0;
	
	if(CPU_REG_PIOSET == ADDRESS_UNDEFINE || CPU_REG_PIORESET == ADDRESS_UNDEFINE)
		return;

	green_LED_counter++;
	if (green_LED_counter>0x35555) /* about 0.1 secs*/
	{
		green_LED_counter=0;

		for(i=1;i<5;i++)
		{
			alive=athrs27_phy_is_link_alive(i);

			if(alive^lan_flag[i-1]) /* ^ is XOR */
			{
				lan_flag[i-1]=alive;
				if(alive)
				{
					ar7240_reg_wr (CPU_REG_PIORESET, 1 << i );
				}
				else
				{
					ar7240_reg_wr (CPU_REG_PIOSET,  1 << i );
				}
			}
		}
		/* printf("lan_flag[1~4]=%d %d %d %d \n",lan_flag[0],lan_flag[1],lan_flag[2],lan_flag[3]); */
	}
}

void green_LED_control_stop(void)
{
	/* LAN LNIK SET */
	ar7240_reg_wr (AR7240_GPIO_OUT_FUNCTION0,  (ar7240_reg_rd(AR7240_GPIO_OUT_FUNCTION0)&0x000000ff ) | ( (42 << 8) | (43 << 16 ) | (44 << 24) ) );//port 1 green
	ar7240_reg_wr (AR7240_GPIO_OUT_FUNCTION1,  (ar7240_reg_rd(AR7240_GPIO_OUT_FUNCTION1) &0xffffff00 ) | ( 45 ) );
}
#endif


int gpio_handle(int gpio_number, int gpio_value)
{
	int i=0;
	uint32_t value=0;
	
	if( gpio_number > 22 || gpio_number < 0  )
	{
		printf ("gpio number is out of range [0~22].\n" );
		return -1;
	}
	
	if( gpio_value!=0 && gpio_value!=1 )
	{
		printf ("set_value is out of range [0~1].\n");
		return -1;
	}
	printf("\n");
	for(i=31;i>=0;i--)
	{
		printf("%02d ", i);
	}

	printf("--------- GPIO Numer\n");
	value=ar7240_reg_rd(CPU_GPIO_OUT_VALUE);
	for(i=31;i>=0;i--)
	{
		printf("%02d ", (value & 1<< i ) >> i);
	}
	printf(" %08x CPU_GPIO_OUT_VALUE\n",ar7240_reg_rd(CPU_GPIO_OUT_VALUE));
	
	if(gpio_value)
	{
		ar7240_reg_wr (CPU_REG_PIOSET,  1 << gpio_number );
	}
	else
	{
		ar7240_reg_wr (CPU_REG_PIORESET, 1 << gpio_number );
	}

	value=ar7240_reg_rd(CPU_GPIO_OUT_VALUE);
	for(i=31;i>=0;i--)
	{
		if(i==gpio_number)
		    printf(" | "); 
		else
			printf("   ");
	}
	printf("\n");
	
	for(i=31;i>=0;i--)
	{
		printf("%02d ", (value & 1<< i ) >> i);
	}
	printf(" %08x CPU_GPIO_OUT_VALUE\n",ar7240_reg_rd(CPU_GPIO_OUT_VALUE));

}

/* If the address is not zero, reading the value from the address.*/
uint32_t read_cpu_value(uint32_t address)
{
	uint32_t value=0;
	
	if(address != ADDRESS_UNDEFINE)
	{
		value=ar7240_reg_rd(address);
	}
    return value;
}

/* If the address is not zero, writing the value to the address.*/
void save_cpu_value(uint32_t init_value, uint32_t result_value, uint32_t address)
{
	int i=0;
	uint32_t value=0;
	
	if( (address == ADDRESS_UNDEFINE) || (init_value == result_value) ) 
		return;

	ar7240_reg_wr (address,  result_value);	
#ifdef GPIO_DEBUG
	for(i=31;i>=0;i--)
	{
		printf("%02d ", i);
	}
	printf("\n\n");

	if(address ==CPU_REG_PIOSET || address == CPU_REG_PIORESET)
	{
		for(i=31;i>=0;i--)
		{
			printf("%02d ", (result_value & 1<< i ) >> i);
		}
		printf("%08x  add:%x \n\n",result_value,address);
	}
	else
	{
		for(i=31;i>=0;i--)
		{
			printf("%02d ", (init_value & 1<< i ) >> i);
		}
		printf("%08x  add:%x\n",init_value,address);
		value=ar7240_reg_rd(address);
		for(i=31;i>=0;i--)
		{
			printf("%02d ", (value & 1<< i ) >> i);
		}
		printf("%08x  add:%x\n\n",value,address);
	}
#endif
}

/* Set the value of GPIO_Mode to CPU. */
void set_gpio_mode(void)
{
    int i=0;
	uint32_t cpu_value=0, mode_value=0;

	mode_value = cpu_value = read_cpu_value(CPU_GPIOMODE_REG);
	for(i=0;i<sizeof(alpha_gpio_mode_table)/sizeof(alpha_gpio_mode_t);i++)
	{
		if(alpha_gpio_mode_table[i].bit_value )
			mode_value |= 1 << alpha_gpio_mode_table[i].bit;
		else
			mode_value &= ~((1<<alpha_gpio_mode_table[i].bit));
	}		
	save_cpu_value(cpu_value,mode_value,CPU_GPIOMODE_REG);
}

uint32_t gpio_IO_value(uint32_t value,int index)
{
    int pin_value=0;
	
	if(alpha_gpio_table[index].pin < GPIO_REGISTER_THRESHOLD ) 
	{
		pin_value=alpha_gpio_table[index].pin;
	}
	else
	{
		pin_value=alpha_gpio_table[index].pin - GPIO_REGISTER_THRESHOLD;
	}

	if(alpha_gpio_table[index].dir == GPIO_OUTPUT)
	{ 
		if(GPIO_SET_OUTPUT_IS_1)
		{ /* If GPIO's direction is GPIO_OUTPUT, set the Corresponding bit to 1 */
			value |= 1 << pin_value;
		}
		else
		{ /* If GPIO's direction is GPIO_OUTPUT, set the Corresponding bit to 0 */
			value &= ~(1<<pin_value);
		}
	}
	else if(alpha_gpio_table[index].dir == GPIO_INPUT)
	{
		if(GPIO_SET_OUTPUT_IS_1)
		{ /* If GPIO's direction is GPIO_INPUT, set the Corresponding bit to 0 */
			value &= ~(1<<pin_value);
		}
		else
		{ /* If GPIO's direction is GPIO_INPUT, set the Corresponding bit to 1 */
			value |= 1 << pin_value;
		}
	}

	return value;
}

/* Set the GPIO's direction(INPUT/OUTPUT) and initial value(pull high/low). */
void set_gpio_IO_and_init_value(void)
{
    int i=0, new_pin=0;
	uint32_t cpu_value=0, cpu_value3924=0, dir_value=0, dir_value_3924=0;
	uint32_t init_high_value=0, init_low_value=0,init_high_value_3924=0,init_low_value_3924=0;
	
	dir_value      = cpu_value     = read_cpu_value(CPU_REG_PIODIR);
	dir_value_3924 = cpu_value3924 = read_cpu_value(CPU_REG_PIODIR3924);

	for(i=0;i<sizeof(alpha_gpio_table)/sizeof(alpha_gpio_table_t);i++)
	{
		if(alpha_gpio_table[i].pin < GPIO_REGISTER_THRESHOLD ) 
		{ /* If the range of gpio number is 0~23, we set the address are 
				CPU_REG_PIODIR/CPU_REG_PIOSET/CPU_REG_PIORESET. */
			dir_value=gpio_IO_value(dir_value,i);
			
			if(alpha_gpio_table[i].init_gpio_value == GPIO_HIGH)
			{ /* If GPIO's initial value is GPIO_HIGH, set the Corresponding bit to 1. */
				init_high_value |= 1 << alpha_gpio_table[i].pin;
			}
			else if(alpha_gpio_table[i].init_gpio_value == GPIO_LOW)
			{ /* If GPIO's initial value is GPIO_LOW, set the Corresponding bit to 1. */
				init_low_value |= 1 << alpha_gpio_table[i].pin;
			}			
		}
		else
		{   /* If the range of gpio number is 24~39, we set the address are 
				CPU_REG_PIODIR3924/CPU_REG_PIOSET3924/CPU_REG_PIORESET3924. */
			new_pin=alpha_gpio_table[i].pin - GPIO_REGISTER_THRESHOLD;
			
			dir_value_3924=gpio_IO_value(dir_value_3924,i);
						
			if(alpha_gpio_table[i].init_gpio_value == GPIO_HIGH)
			{
				init_high_value_3924 |= 1 << new_pin;
			}
			else if(alpha_gpio_table[i].init_gpio_value == GPIO_LOW)
			{
				init_low_value_3924 |= 1 << new_pin;
			}
		}
	}
	save_cpu_value(cpu_value,dir_value,CPU_REG_PIODIR);
	save_cpu_value(cpu_value3924,dir_value_3924,CPU_REG_PIODIR3924);
	save_cpu_value(0,init_high_value,CPU_REG_PIOSET);
	save_cpu_value(0,init_low_value,CPU_REG_PIORESET);
	save_cpu_value(0,init_high_value_3924,CPU_REG_PIOSET3924);
	save_cpu_value(0,init_low_value_3924,CPU_REG_PIORESET3924);
}

int is_on=0,STATUS_LED_counter=0;
void toggle_activity(void)
{
	if(CPU_REG_PIOSET == ADDRESS_UNDEFINE || CPU_REG_PIORESET == ADDRESS_UNDEFINE)
		return;

	STATUS_LED_counter++;
	if (STATUS_LED_counter>0x215555) /* about 2 secs*/
	{
		STATUS_LED_counter=0;
		is_on^=1;	

		if (is_on==1)
		{ /* light on  (POWER, orange led). Then, when running kernel in runtime code we will light it on jack +++ */
		    //	printf(" : light on\n");
			ar7240_reg_wr (CPU_REG_PIOSET,  1 << GPIO_POWER_LED );
		}
		else
		{ /* light off  (POWER, orange led). Then, when running kernel in runtime code we will light it on jack +++ */	
			//	printf(" : light off\n");
			ar7240_reg_wr (CPU_REG_PIORESET, 1 << GPIO_POWER_LED );
		}
	}	

}

void toggle_activity_light_on(void)
{
	if(CPU_REG_PIOSET == ADDRESS_UNDEFINE)
		return;
		
	ar7240_reg_wr (CPU_REG_PIOSET,  1 << GPIO_POWER_LED );
}


int force_into_httpsvr(void)
{
	/* printf("\noolong: ar7240_reg_rd(AR7240_GPIO_IN)=%x \n",ar7240_reg_rd(AR7240_GPIO_IN)); */
	if( AR7240_GPIO_IN != ADDRESS_UNDEFINE )
	{
	   /* Use freset button */
		if( (ar7240_reg_rd(AR7240_GPIO_IN) & ( 1 << GPIO_RESET_BUTTON ) ) == 0 )
		{
			printf("\nFRESET button pressed, force into HTTP server.\n");
			return 1;
		}
	}
	return 0;
}

void gpio_init(void)
{

	set_gpio_mode();
	set_gpio_IO_and_init_value();
	gpio_init_extra();

}
